﻿// -------------------------------------------------------------------------------------------------------------------- 
// <copyright company="{Company}">
//  This file is part of Just-In WhitePages.
//
//   Just-In WhitePages is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Just-In WhitePages is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Just-In WhitePages.  If not, see <http://www.gnu.org/licenses/>.
// </copyright> 
// -------------------------------------------------------------------------------------------------------------------- 

using System;
using System.Collections.Generic;
using System.DirectoryServices;
using System.Configuration;
using System.Text;
using System.Data;
using System.Xml;
using System.Security;
using System.Security.Principal;
using System.Web;
using WhitePages.Directory;
using System.Text.RegularExpressions;

public enum FilterOperator
{
    Is=1,
    Exactly=2,
    StartsWith=3,
    EndsWith=4
}

public class DirectoryLookup
{

    public static DirectoryEntry OpenDirectoryEntry(string baseDn)
    {
        string userName = ConfigurationManager.AppSettings["DirectoryUserID"];
        string password = ConfigurationManager.AppSettings["DirectoryPassword"];
        if (userName != null && userName.Length > 0)
            return new DirectoryEntry(string.Concat("LDAP://", WhitePages.Settings.ServerName, "/", baseDn), userName, password, AuthenticationTypes.Secure);
        else
            return new DirectoryEntry(string.Concat("LDAP://", WhitePages.Settings.ServerName, "/", baseDn));
    }

    public static DirectoryEntry OpenDirectoryEntry(string baseDn, string userName, string password)
    {
        return new DirectoryEntry(string.Concat("LDAP://", WhitePages.Settings.ServerName, "/", baseDn), userName, password, AuthenticationTypes.Secure);
    }

    
    public static bool AuthenticateUser(string dn, string password)
    {
        DirectoryEntry de = OpenDirectoryEntry(dn);
        string userID = GetProperty(de, "userPrincipalName");
        if (userID.Length <= 0)
            userID = GetProperty(de, "sAMAccountName");
        de = OpenDirectoryEntry(dn,userID, password);
        try
        {
            object cn = de.Properties["cn"].Value;
            return true;
        }
        catch 
        {
            return false;
        }
    }
    

    public static string ValidateQuery(string query)
    {
        StringBuilder sb = new StringBuilder();
        Regex rgex = new Regex(WhitePages.Settings.ValidQueryCharacters);
        MatchCollection mc = rgex.Matches(query);

        foreach (Match m in mc)
        {
            sb.Append(m.Value);
            sb.Append(" ");
        }
        return sb.ToString().Trim();
    }

    public static string ParseNameQuery(string query)
    {        
        StringBuilder filter = new StringBuilder();
        string baseFilter = ConfigurationManager.AppSettings["BaseFilter"];
        string[] parts;

        query = ValidateQuery(query);
        parts = query.Split(' ');

        filter.Append("(");
        if (!string.IsNullOrEmpty(baseFilter))
        {            
            filter.Append("&");
            filter.Append(baseFilter);
            filter.Append("(");
        }

        if (parts.Length == 2 && WhitePages.Settings.NamingPattern== WhitePages.NamingPatterns.FirstLast)
            filter.AppendFormat("|(cn=*{0}* {1})(&(givenName={0}*)(sn={1}*))", parts);
        else if (parts.Length == 2 && WhitePages.Settings.NamingPattern == WhitePages.NamingPatterns.LastCommaFirst)
            filter.AppendFormat("|(cn={1}, *{0}*)(&(givenName={1}*)(sn={0}*))", parts);
        else if (query.Contains("*"))
            filter.AppendFormat("|(givenName={0})(sn={0})", query);
        else if (query.Length > 0)
            filter.AppendFormat("(cn=*{0}*)", query);
        else
            filter.Append("(cn=*)");
        
        filter.Append("))");

        return filter.ToString();
    }

    public static string AddBaseFilter(string ldapFilter)
    {
        StringBuilder filter = new StringBuilder();
        string baseFilter = ConfigurationManager.AppSettings["BaseFilter"];

        filter.Append("(");
        if (!string.IsNullOrEmpty(baseFilter))
        {
            filter.Append("&");
            filter.Append(baseFilter);
            filter.Append("(");
        }

        filter.Append("(");
        filter.Append(ldapFilter);
        filter.Append(")");

        filter.Append("))");

        return filter.ToString();
    }

    public static string DecorateFilterValue(FilterOperator oper, string value)
    {
        if (oper == FilterOperator.Is)
            return string.Format("*{0}*", value);
        else if (oper == FilterOperator.Exactly)
            return string.Format("{0}", value);
        else if (oper == FilterOperator.StartsWith)
            return string.Format("{0}*", value);
        else if (oper == FilterOperator.EndsWith)
            return string.Format("*{0}", value);
        else
            return string.Format("*{0}*",value);
    }
    

    public static DataView FindAll(string filter, string sort, string [] propertiesToLoad)
    {
        DirectoryEntry de = OpenDirectoryEntry(WhitePages.Settings.SearchRoot);
        DirectorySearcher ds = new DirectorySearcher(de, filter);
        SearchResultCollection results;

        ds.PropertiesToLoad.AddRange(propertiesToLoad);
        if (!ds.PropertiesToLoad.Contains("distinguishedName"))
            ds.PropertiesToLoad.Add("distinguishedName");
        if (!ds.PropertiesToLoad.Contains("cn"))
            ds.PropertiesToLoad.Add("cn");

        ds.Sort.PropertyName = sort.Split(',')[0];
        ds.SizeLimit = WhitePages.Settings.ResultLimit;

        results = ds.FindAll();
        
        return new DataView(ConvertToDataTable(results), null, sort, DataViewRowState.CurrentRows); 
    }

    public static DataView Find(string[] propertiesToLoad, string filter, string sort, out bool overLimit)
    {
        DirectoryEntry de = OpenDirectoryEntry(WhitePages.Settings.SearchRoot);
        DirectorySearcher ds = new DirectorySearcher(de, AddBaseFilter(filter));
        SearchResultCollection results;

        ds.PropertiesToLoad.AddRange(propertiesToLoad);
        if (!ds.PropertiesToLoad.Contains("distinguishedName"))
            ds.PropertiesToLoad.Add("distinguishedName");
        if (!ds.PropertiesToLoad.Contains("cn"))
            ds.PropertiesToLoad.Add("cn");

        ds.Sort.PropertyName = sort.Split(',')[0];
        ds.SizeLimit = WhitePages.Settings.ResultLimit;

        ds.SizeLimit = WhitePages.Settings.ResultLimit + 1;

        results = ds.FindAll();

        if (results.Count > WhitePages.Settings.ResultLimit)
            overLimit = true;
        else
            overLimit = false;

        return new DataView(ConvertToDataTable(results), null, sort, DataViewRowState.CurrentRows);
    }

    public static DataView FindByName(string[] propertiesToLoad,string name, string sort, out bool overLimit)
    {
        DirectoryEntry de = OpenDirectoryEntry(WhitePages.Settings.SearchRoot);
        DirectorySearcher ds = new DirectorySearcher(de, ParseNameQuery(name));
        SearchResultCollection results;

        ds.PropertiesToLoad.AddRange(propertiesToLoad);
        if (!ds.PropertiesToLoad.Contains("distinguishedName"))
            ds.PropertiesToLoad.Add("distinguishedName");
        if (!ds.PropertiesToLoad.Contains("cn"))
            ds.PropertiesToLoad.Add("cn");

        ds.Sort.PropertyName = sort.Split(',')[0];

        ds.SizeLimit = WhitePages.Settings.ResultLimit + 1;
                
        results = ds.FindAll();

        if (results.Count > WhitePages.Settings.ResultLimit)
            overLimit = true;
        else
            overLimit = false;

        return new DataView(ConvertToDataTable(results), null, sort, DataViewRowState.CurrentRows); 
    }

    public static DataView FindByName(string[] propertiesToLoad, string name, string sort)
    {
        bool overLimit;
        return FindByName(propertiesToLoad, name, sort, out overLimit);
    }


    public static DirectoryEntry GetRecord(string dn)
    {
        DirectoryEntry de = OpenDirectoryEntry(dn);               
        return de;
    }

    public static DirectoryEntry GetRecord(string dn,string userName, string password)
    {
        DirectoryEntry de = OpenDirectoryEntry(dn, userName, password);
        return de;
    }

    public static string GetProperty(DirectoryPropertyAccessor dpa, string propertyName)
    {
        PropertyInfo pi = GetPropertyInfo(propertyName);
        switch (pi.Type)
        {
            case "FileTime":
                return (pi.FormatString.Length > 0) ? dpa.GetDate(propertyName).ToString(pi.FormatString): dpa.GetDate(propertyName).ToString();
            case "FileTimeUtc":
                return (pi.FormatString.Length > 0) ? dpa.GetUtcDate(propertyName).ToString(pi.FormatString) : dpa.GetUtcDate(propertyName).ToString();
            case "distinguishedName":
                return GetProperty(dpa.GetDirectoryEntry(propertyName), pi.DisplayProperty) ;
            default:
                return dpa.GetString(propertyName);
        }
    }

    public static string GetProperty(SearchResult sr, string propertyName)
    {
        return GetProperty(new DirectoryPropertyAccessor(sr), propertyName);
    }

    public static string GetProperty(DirectoryEntry sr, string propertyName)
    {
        return GetProperty(new DirectoryPropertyAccessor(sr), propertyName);       
    }


    public static void SetProperty(DirectoryEntry de, string propertyName, object value)
    {
        DirectoryPropertyAccessor dpa = new DirectoryPropertyAccessor(de);
        dpa.SetValue(propertyName, value);
    }

    private static DataTable ConvertToDataTable(SearchResultCollection sr)
    {
        DataTable dt = new DataTable("SearchResults");

        foreach (string propName in sr.PropertiesLoaded)
        {
            dt.Columns.Add(propName);
        }

        dt.BeginLoadData();
        foreach (SearchResult r in sr)
        {
            AddDataRow(r, dt);
        }
        dt.EndLoadData();

        return dt;
    }

    private static void AddDataRow(SearchResult sr, DataTable dt)
    {
        DataRow row = dt.NewRow();
        DirectoryPropertyAccessor dpa = new DirectoryPropertyAccessor(sr);        
        
        foreach (DataColumn dc in dt.Columns)
        {
            PropertyInfo pi = GetPropertyInfo(dc.ColumnName);
            switch (pi.Type)
            {
                case "FileTime":
                    row[dc.ColumnName] = (pi.FormatString.Length > 0) ? dpa.GetDate(dc.ColumnName).ToString(pi.FormatString) : dpa.GetDate(dc.ColumnName).ToString();
                    break;
                case "FileTimeUtc":
                    row[dc.ColumnName] = (pi.FormatString.Length > 0) ? dpa.GetUtcDate(dc.ColumnName).ToString(pi.FormatString) : dpa.GetUtcDate(dc.ColumnName).ToString();
                    break;
                case "distinguishedName":
                    row[dc.ColumnName] = GetProperty(dpa.GetDirectoryEntry(dc.ColumnName), pi.DisplayProperty);
                    break;
                default:
                    row[dc.ColumnName] = dpa.GetValue(dc.ColumnName,string.Empty);
                    break;
            }

            //if (firstLetterIndexPropName != null && firstLetterIndexPropName.Length > 0 && string.Compare(dc.ColumnName,firstLetterIndexPropName, StringComparison.OrdinalIgnoreCase)==0)
            //{
            //    firstLetterIndex.Add(row[dc.ColumnName].ToString().Substring(0));
            //}

        }
        dt.Rows.Add(row);
    }

    public static PropertyInfo GetPropertyInfo(string propertyName)
    {
        if (WhitePages.Settings.PropertyNames.ContainsKey(propertyName))
            return WhitePages.Settings.PropertyNames[propertyName];
        else
            return new PropertyInfo(propertyName, string.Empty, string.Empty,string.Empty,string.Empty,string.Empty,string.Empty,null);
    }

    public static bool TryCommitChanges(DirectoryEntry de)
    {
        try
        {
            de.CommitChanges();
            return true;
        }
        catch (UnauthorizedAccessException ex)
        {
            return false;
        }
    }
}